package com.daniminguet.videogameadvisor;

import java.util.ArrayList;

import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.daniminguet.videogameadvisor.model.Videogame;
import com.daniminguet.videogameadvisor.service.VideogameQueryService;
import com.daniminguet.videogameadvisor.service.VideogameService;

@Component
public class VideogameAdvisorRunApp {
	@Autowired
	private VideogameService videogameService;
	@Autowired
	private VideogameQueryService videogameQueryService;
	@Autowired
	private VideogameAdvisorHelp help;
	
	public void run(String[] args) {
		if(args.length < -1) {
			System.out.println("Error de sintaxis");
			System.out.println(help.getHelp());
		} else if (args.length == 1){
			switch(args[0].toLowerCase()) {
				case "-lg":
					videogameService.findAllGenres().forEach(System.out::println);
					break;
				case "-h":
					System.out.println(help.getHelp());
					break;
				default:
					System.out.println("Error de sintaxis");
					System.out.println(help.getHelp());
			}
		} else if (args.length % 2 != 0) {
			System.out.println("Error de sintaxis");
			System.out.println(help.getHelp());
		} else if (args.length > 8) {
			System.out.println("Error de sintaxis");
			System.out.println(help.getHelp());
		} else if (args.length == 0) {
			System.out.println("No se ha pasado ningún parámetro");
			System.out.println(help.getHelp());
		} else {
			List<String[]> argumentos = new ArrayList<>();
			
			for(int i = 0; i < args.length; i += 2) {
				argumentos.add(new String[] {args[i], args[i + 1]});
			}
			
			boolean error = false;
			
			for(String[] argumento : argumentos) {
				switch (argumento[0].toLowerCase()) {
				case "-ag":
					videogameQueryService.anyGenre(argumento[1].split(","));
					break;
				case "-tg":
					videogameQueryService.allGenres(argumento[1].split(","));
					break;
				case "-y":
					videogameQueryService.year(argumento[1]);
					break;
				case "-b":
					String[] years = argumento[1].split(",");
					videogameQueryService.betweenYears(years[0], years[1]);
					break;
				case "-t":
					videogameQueryService.titleContains(argumento[1]);
					break;
				default: error = true;
						 System.out.println("Error de sintaxis");
						 System.out.println(help.getHelp());
				}
			}
			
			if (!error) {
				Collection<Videogame> result = videogameQueryService.exec();
				System.out.printf("%s\t%-50s\t%s\t%s\t%s\t%s\n","ID","Nombre", "Desarrollador", "Géneros", "Fecha de salida", "Plataformas disonibles");
				if (result != null) {
					result.forEach(vg -> System.out.printf("%s\t%-50s\t%s\t%s\t%s\t%s\n", 
							vg.getId(), vg.getNombre(), vg.getDesarrollador(), 
							vg.getGeneros().stream().collect(Collectors.joining(", ")), 
							vg.getFechaSalida(), vg.getPlataformas()));
				} else {
					System.out.println("No hay videojuegos que cumplan esos criterios. Lo sentimos");
				}
			}
		}
	}
}
