package com.daniminguet.videogameadvisor.service;

import java.time.LocalDate;
import java.util.Arrays;
import java.util.Collection;
import java.util.function.Predicate;
import java.util.stream.Collectors;

import javax.annotation.PostConstruct;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.daniminguet.videogameadvisor.dao.VideogameDao;
import com.daniminguet.videogameadvisor.model.Videogame;

@Service
public class VideogameQueryServiceImpl implements VideogameQueryService {
	@Autowired
	private VideogameDao dao;
	
	private Predicate<Videogame> predicate;
	
	@PostConstruct
	public void init() {
		predicate = null;
	}

	@Override
	public Collection<Videogame> exec() {
		//@formatter:off
		return dao.findAll()
				.stream()
				.filter(predicate)
				.collect(Collectors.toList());
		//@formatter:on
	}

	@Override
	public VideogameQueryService anyGenre(String... genres) {
		Predicate<Videogame> pAnyGenre = (vg -> Arrays.stream(genres).anyMatch(vg.getGeneros()::contains));
		predicate = (predicate == null) ? pAnyGenre : predicate.and(pAnyGenre);
		return this;
	}

	@Override
	public VideogameQueryService allGenres(String... genres) {
		Predicate<Videogame> pAllGenre = (vg -> Arrays.stream(genres).allMatch(vg.getGeneros()::contains));
		predicate = (predicate == null) ? pAllGenre : predicate.and(pAllGenre);
		return this;
	}

	@Override
	public VideogameQueryService year(String year) {
		Predicate<Videogame> pYear = (vg -> vg.getFechaSalida().substring(vg.getFechaSalida().length()-4, vg.getFechaSalida().length()-1).equalsIgnoreCase(year));
		predicate = (predicate == null) ? pYear : predicate.and(pYear);
		return this;
	}

	@Override
	public VideogameQueryService betweenYears(String from, String to) {
		Predicate<Videogame> pBetweenYears = (vg -> {
			LocalDate fromYear = LocalDate.of(Integer.parseInt(from), 1, 1);
			LocalDate toYear = LocalDate.of(Integer.parseInt(to), 1, 3);
			LocalDate videogameYear = LocalDate.of(Integer.parseInt(vg.getFechaSalida().substring(vg.getFechaSalida().length()-4, vg.getFechaSalida().length()-1)), 1, 2);
			
			return videogameYear.isAfter(fromYear) && videogameYear.isBefore(toYear);
		});
		
		predicate = (predicate == null) ? pBetweenYears : predicate.and(pBetweenYears);
		return this;
	}

	@Override
	public VideogameQueryService titleContains(String title) {
		Predicate<Videogame> pTitleContains = (vg -> vg.getNombre().toLowerCase().contains(title.toLowerCase()));
		predicate = (predicate == null) ? pTitleContains : predicate.and(pTitleContains);
		return this;
	}

}
