package com.daniminguet.videogameadvisor.service;

import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.daniminguet.videogameadvisor.dao.VideogameDao;
import com.daniminguet.videogameadvisor.model.Videogame;

@Service
public class VideogameService {
	@Autowired
	VideogameDao videogameDao;

	@Autowired
	VideogameQueryService queryService;

	public Collection<String> findAllGenres() {
		List<String> result = null;

		// @formatter:off
		result = videogameDao.findAll()
				.stream()
				.map(vg -> vg.getGeneros())
				.flatMap(list -> list.stream())
				.distinct()
				.sorted()
				.collect(Collectors.toList());

		// @formatter:on

		return result;
	}

	public Collection<Videogame> findByAnyGenre(String... genres) {
		return queryService.anyGenre(genres).exec();
	}

	public Collection<Videogame> findByAllGenres(String... genres) {
		return queryService.allGenres(genres).exec();
	}

	public Collection<Videogame> findByYear(String year) {
		return queryService.year(year).exec();
	}

	public Collection<Videogame> findBetweenYears(String from, String to) {
		return queryService.betweenYears(from, to).exec();
	}

	public Collection<Videogame> findByTitleContains(String title) {
		return queryService.titleContains(title).exec();
	}

	public Collection<Videogame> findAll() {
		return videogameDao.findAll();
	}
}
